searchData={"items":[{"type":"task","title":"mix rambla.rabbit.exchange","doc":"Mix task to deal with exchanges in the target RabbitMQ.\n\nThis is helpful to orchestrate target RabbitMQ when deploying\nto docker. Allows to create and delete the exchange.\n\nLoads the setting from `config :rambla, :amqp` if no connection\nis provided in parameters.","ref":"Mix.Tasks.Rambla.Rabbit.Exchange.html"},{"type":"task","title":"Command line options - mix rambla.rabbit.exchange","doc":"* -c - the connection string\n  * -o - the list of options without spaces, separated by comma","ref":"Mix.Tasks.Rambla.Rabbit.Exchange.html#module-command-line-options"},{"type":"task","title":"Options - mix rambla.rabbit.exchange","doc":"","ref":"Mix.Tasks.Rambla.Rabbit.Exchange.html#module-options"},{"type":"task","title":"Options for create - mix rambla.rabbit.exchange","doc":"* **`type`** - One of four possible values below. Defaults to `:direct`.\n    * `direct`\n    * `fanout`\n    * `topic`\n    * `headers`\n  * `durable` - If set, keeps the Exchange between restarts of the broker;\n  * `auto_delete` - If set, deletes the Exchange once all queues\n    unbind from it;\n  * `passive` - If set, returns an error if the Exchange does not\n    already exist;\n  * `internal` - If set, the exchange may not be used directly by\n    publishers, but only when bound to other exchanges. Internal exchanges are used to construct wiring that is not visible to applications.\n  * `no_wait` - If set, the declare operation is asynchronous.\n    Defaults to false.\n  * `arguments` - A list of arguments to pass when declaring\n    (of type AMQP.arguments/0). See the README for more information. Defaults to [].","ref":"Mix.Tasks.Rambla.Rabbit.Exchange.html#module-options-for-create"},{"type":"task","title":"Options for delete - mix rambla.rabbit.exchange","doc":"* `if_unused` - If set, the server will only delete the exchange\n    if it has no queue bindings.\n  * `no_wait` - If set, the delete operation is asynchronous.","ref":"Mix.Tasks.Rambla.Rabbit.Exchange.html#module-options-for-delete"},{"type":"type","title":"Mix.Tasks.Rambla.Rabbit.Exchange.command/0","doc":"","ref":"Mix.Tasks.Rambla.Rabbit.Exchange.html#t:command/0"},{"type":"task","title":"mix rambla.rabbit.queue","doc":"Mix task to deal with queues in the target RabbitMQ.\n\nThis is helpful to orchestrate target RabbitMQ when deploying\nto docker. Allows to create, delete, purge and query status of\nthe queue. Also, `bind` and `unbind` commands are supported,\nboth require `exchange:...` option to be passed.\n\nLoads the setting from `config :rambla, :amqp` if no connection\nis provided in parameters.","ref":"Mix.Tasks.Rambla.Rabbit.Queue.html"},{"type":"task","title":"Command line options - mix rambla.rabbit.queue","doc":"* -c - the connection string\n  * -o - the list of options without spaces, separated by comma","ref":"Mix.Tasks.Rambla.Rabbit.Queue.html#module-command-line-options"},{"type":"task","title":"Options - mix rambla.rabbit.queue","doc":"","ref":"Mix.Tasks.Rambla.Rabbit.Queue.html#module-options"},{"type":"task","title":"Options for create - mix rambla.rabbit.queue","doc":"* `durable` - If set, keeps the Queue between restarts\n    of the broker. Defaults to false.\n  * `auto_delete` - If set, deletes the Queue once all\n    subscribers disconnect. Defaults to false.\n  * `exclusive` - If set, only one subscriber can consume\n    from the Queue. Defaults to false.\n  * `passive` - If set, raises an error unless the queue\n    already exists. Defaults to false.\n  * `no_wait` - If set, the declare operation is asynchronous.\n    Defaults to false.\n  * `arguments` - A list of arguments to pass when declaring\n    (of type AMQP.arguments/0). See the README for more information. Defaults to [].","ref":"Mix.Tasks.Rambla.Rabbit.Queue.html#module-options-for-create"},{"type":"task","title":"Options for delete - mix rambla.rabbit.queue","doc":"* `if_unused` - If set, the server will only delete the queue\n    if it has no consumers. If the queue has consumers, it’s\n    not deleted and an error is returned.\n  * `if_empty` - If set, the server will only delete the queue\n    if it has no messages.\n  * `no_wait` - If set, the delete operation is asynchronous.","ref":"Mix.Tasks.Rambla.Rabbit.Queue.html#module-options-for-delete"},{"type":"type","title":"Mix.Tasks.Rambla.Rabbit.Queue.command/0","doc":"","ref":"Mix.Tasks.Rambla.Rabbit.Queue.html#t:command/0"},{"type":"module","title":"Rambla","doc":"Interface for the message publishing through `Rambla`.\n\n`Rambla` maintains connection pools with `Finitomata.Pool` for each service.\n\nThe typical config for `Rambla` service follows the pattern introduced by\n  `AMQP` library:\n\n```elixir\n‹service›: [\n  connections: [\n    ‹connection_name›: [‹key_1›: ‹value_1›, …]\n  ],\n  channels: [\n    ‹channel_name›: [\n      connection: ‹connection_name›,\n      options: [‹key_1›: ‹value_1›, …]\n    ]\n  ]\n]\n```\n\nAdditional option, one might pass to the channel config, would be explicit handlers\n  for failures and success calls (by default the former prints the warning and retries\n  until the maximum count of retries reached, and then calls `on_fatal/2` callback,\n  and the latter logs a debug message.)\n\n```elixir\n  channels: [\n    chan_1: [\n      connection: :conn_1,\n      options: [\n        callbacks: [\n          on_success: fn result -> IO.inspect(result, label: \"on_success\") && :ok end]\n        ]]]]\n```\n\n---\n\nTo start pools, simply embed `Rambla` into the supervision tree, it’d\n  start a supervisor with children for all the configured services.\n\nThe configuration of the service implies all the `Rambla`’s code for it will\n  be compiled, but the dependency itself must be added to the `deps` section\n  of the `Mix.Project` file.\n\nThe excerpt from the `Rambla.MixProject` itself follows\n\n```elixir\n  # optional backends\n  {:amqp, \"~> 3.0\", optional: true},\n  {:redix, \"~> 1.0\", optional: true},\n  {:pillar, \"~> 0.39\", optional: true},\n  {:gen_smtp, \"~> 0.4 or ~> 1.0\", optional: true},\n  {:telemetria, \"~> 0.4 or ~> 1.0\", optional: true},\n\n  # s3\n  {:ex_aws, \"~> 2.1\", optional: true},\n  {:ex_aws_s3, \"~> 2.0\", optional: true},\n  {:ex_aws_sts, \"~> 2.0\", optional: true},\n  {:hackney, \"~> 1.9\", optional: true},\n  {:sweet_xml, \"~> 0.6\", optional: true},\n  {:configparser_ex, \"~> 4.0\", optional: true},\n```\n\n---\n\nChannel names are used across connections to publish messages.\n  `Rambla.publish(:channel_1, message)` would publish the message to all channels\n  named `channel_1`.","ref":"Rambla.html"},{"type":"function","title":"Rambla.channels/0","doc":"Returns a map `%{‹service› => [‹channels›]}`","ref":"Rambla.html#channels/0"},{"type":"function","title":"Rambla.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Rambla.html#child_spec/1"},{"type":"function","title":"Rambla.connections/0","doc":"Returns a map `%{‹channel› => [‹connection›]}`","ref":"Rambla.html#connections/0"},{"type":"function","title":"Rambla.pools/0","doc":"Returns the currently active pools","ref":"Rambla.html#pools/0"},{"type":"function","title":"Rambla.publish/3","doc":"Publishes the message to the target channels. The message structure depends on\nthe destination. For `RabbitMQ` is might be whatever, for `Smtp` it expects\nto have `to:`, `subject:` and `body:` fields.","ref":"Rambla.html#publish/3"},{"type":"function","title":"Rambla.publish_synch/2","doc":"Publishes the message to the destination synchronously, avoiding the pool.","ref":"Rambla.html#publish_synch/2"},{"type":"function","title":"Rambla.publish_synch/3","doc":"Publishes the message to the destination synchronously, avoiding the pool.\nUnlike `publish_synch/2`, allows to specify additional options per request.","ref":"Rambla.html#publish_synch/3"},{"type":"function","title":"Rambla.raw/2","doc":"Executes any arbitrary function in the context of one of workers in the\nrespective connection pool for the target.\n\nThe function would receive a pid of the connection process.","ref":"Rambla.html#raw/2"},{"type":"function","title":"Rambla.services/0","doc":"Returns a list of all the configured services (connections)","ref":"Rambla.html#services/0"},{"type":"function","title":"Rambla.start_link/1","doc":"Starts the supervisor with all the configured services","ref":"Rambla.html#start_link/1"},{"type":"function","title":"Rambla.start_pools/0","doc":"Starts the pools configured in the `config.exs` / `releases.exs` file.\n\nThis call is equivalent to `start_pools(Application.get_env(:rambla, :pools))`.","ref":"Rambla.html#start_pools/0"},{"type":"function","title":"Rambla.start_pools/1","doc":"Starts the pools as specified by options (`map()` or `keyword()`)","ref":"Rambla.html#start_pools/1"},{"type":"behaviour","title":"Rambla.Handler","doc":"Default handler scaffold for all the connections.\n\n> ### `use Rambla.Handler` {: .info}\n>\n> When you `use Rambla.Handler`, the `Rambla.Handler` module will\n> do the following things for your module:\n>\n> - implement `@behaviour Finitomata.Pool.Actor` where `actor/2` will\n>   delegate to `handle_publish/3` expected to be implemented by this module,\n>   and overridable `on_result/2` and `on_error/2` will have a reasonable\n>   default implementation (debug for the former and warn and retry for the latter)\n> - set `@behaviour Rambla.Handler` to invite you to implement real publishing\n>   handler as `handle_publish/3`","ref":"Rambla.Handler.html"},{"type":"behaviour","title":"Example - Rambla.Handler","doc":"```elixir\ndefmodule Rambla.Handler.Some do\n  use Rambla.Handler\n\n  @impl Rambla.Handler\n  def config do\n    [\n      connections: [\n        local_conn: [url: \"amqp://guest:guest@localhost:5672\"],\n      ],\n      channels: [\n        chan_1: [connection: :local_conn]\n      ]\n    ]\n  end\n\n  @impl Rambla.Handler\n  def handle_publish(payload, options, %{connection: conn} = state) do\n    SomeImpl.publish(conn, payload, options)\n  end \nend\n```","ref":"Rambla.Handler.html#module-example"},{"type":"type","title":"Rambla.Handler.callback/0","doc":"The callback function to be passed to `Rambla.publish/3`","ref":"Rambla.Handler.html#t:callback/0"},{"type":"type","title":"Rambla.Handler.channel_name/0","doc":"The channel name","ref":"Rambla.Handler.html#t:channel_name/0"},{"type":"callback","title":"Rambla.Handler.config/0","doc":"The callback to get to the configuration","ref":"Rambla.Handler.html#c:config/0"},{"type":"type","title":"Rambla.Handler.connection_name/0","doc":"The connection name","ref":"Rambla.Handler.html#t:connection_name/0"},{"type":"callback","title":"Rambla.Handler.external_servers/1","doc":"If specified, these services will be started before pools under `:rest_for_one`","ref":"Rambla.Handler.html#c:external_servers/1"},{"type":"callback","title":"Rambla.Handler.handle_publish/3","doc":"The callback to be implemented by the consumer of this code","ref":"Rambla.Handler.html#c:handle_publish/3"},{"type":"callback","title":"Rambla.Handler.on_fatal/2","doc":"The callback to be called when retries exhausted","ref":"Rambla.Handler.html#c:on_fatal/2"},{"type":"type","title":"Rambla.Handler.resolution/0","doc":"The allowed callback resolution","ref":"Rambla.Handler.html#t:resolution/0"},{"type":"module","title":"Rambla.Services.Clickhouse.Conn","doc":"","ref":"Rambla.Services.Clickhouse.Conn.html"},{"type":"function","title":"Rambla.Services.Clickhouse.Conn.async_insert/3","doc":"","ref":"Rambla.Services.Clickhouse.Conn.html#async_insert/3"},{"type":"function","title":"Rambla.Services.Clickhouse.Conn.async_insert_to_table/3","doc":"","ref":"Rambla.Services.Clickhouse.Conn.html#async_insert_to_table/3"},{"type":"function","title":"Rambla.Services.Clickhouse.Conn.async_query/3","doc":"","ref":"Rambla.Services.Clickhouse.Conn.html#async_query/3"},{"type":"function","title":"Rambla.Services.Clickhouse.Conn.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Rambla.Services.Clickhouse.Conn.html#child_spec/1"},{"type":"function","title":"Rambla.Services.Clickhouse.Conn.init/1","doc":"Callback implementation for  GenServer.init/1 .","ref":"Rambla.Services.Clickhouse.Conn.html#init/1"},{"type":"function","title":"Rambla.Services.Clickhouse.Conn.insert/3","doc":"","ref":"Rambla.Services.Clickhouse.Conn.html#insert/3"},{"type":"function","title":"Rambla.Services.Clickhouse.Conn.insert_to_table/3","doc":"","ref":"Rambla.Services.Clickhouse.Conn.html#insert_to_table/3"},{"type":"function","title":"Rambla.Services.Clickhouse.Conn.query/3","doc":"","ref":"Rambla.Services.Clickhouse.Conn.html#query/3"},{"type":"function","title":"Rambla.Services.Clickhouse.Conn.select/3","doc":"","ref":"Rambla.Services.Clickhouse.Conn.html#select/3"},{"type":"function","title":"Rambla.Services.Clickhouse.Conn.start_link/1","doc":"","ref":"Rambla.Services.Clickhouse.Conn.html#start_link/1"},{"type":"module","title":"Rambla.Handlers.Amqp","doc":"Default handler for _AMQP_ connections. For this handler to work properly,\n  one must include and start `:amqp` application with the config like\n\n```elixir\nconfig :amqp,\n  connections: [\n    local_conn: [url: \"amqp://guest:guest@localhost:5672\"],\n  ],\n  channels: [\n    chan_1: [connection: :local_conn]\n  ]\n\n# Then you can access the connection/channel via `Rambla.Handlers.Amqp` as\n\nRambla.Handlers.Amqp.publish(:chan_1, %{message: %{foo: 42}, exchange: \"rambla\"})\n```\n\n---","ref":"Rambla.Handlers.Amqp.html"},{"type":"module","title":"Known Options - Rambla.Handlers.Amqp","doc":"- `:exchange` (default: `\"\"`)\n- `:declare?` (default: false)\n- `:routing_key`, (default: `\"\"`)\n- `:channel_provider` (default: `AMQP.Application`)\n- `:channel_publisher` (default: `AMQP.Basic`)","ref":"Rambla.Handlers.Amqp.html#module-known-options"},{"type":"function","title":"Rambla.Handlers.Amqp.children_specs/1","doc":"The list of [`child_spec`](https://hexdocs.pm/elixir/Supervisor.html#t:child_spec/0) returned\n  to be embedded into a supervision tree.\n\nKnown options:\n\n- `connection_options` — a `keyword()` or a function of arity one, which is to receive\n  channel names and return connection options as a list\n- `count` — the number of workers in the pool\n- `child_opts` — the options to be passed to the worker’s spec (you won’t need those)","ref":"Rambla.Handlers.Amqp.html#children_specs/1"},{"type":"function","title":"Example - Rambla.Handlers.Amqp.children_specs/1","doc":"```elixir\nRambla.Handlers.Redis.children_specs(\n  connection_options: [exchange: \"amq.direct\"], count: 3)\n```","ref":"Rambla.Handlers.Amqp.html#children_specs/1-example"},{"type":"function","title":"Rambla.Handlers.Amqp.extract_options/2","doc":"","ref":"Rambla.Handlers.Amqp.html#extract_options/2"},{"type":"function","title":"Rambla.Handlers.Amqp.publish/3","doc":"An interface to publish messages using the FSM pool.\n\nThe `id` is the specific to an implementation,\n  for `Amqp` it’d be the channel name, for instance.\n\nThe second parameter would be a payload, or, if the backend supports it,\n  the function of arity one, which would receive back the connection `pid`.","ref":"Rambla.Handlers.Amqp.html#publish/3"},{"type":"function","title":"Example - Rambla.Handlers.Amqp.publish/3","doc":"```elixir\nRambla.Handlers.Amqp.publish :channel_name, %{foo: :bar}\n```","ref":"Rambla.Handlers.Amqp.html#publish/3-example"},{"type":"function","title":"Rambla.Handlers.Amqp.start_link/1","doc":"The entry point: this would start a supervisor with all the pools and stuff","ref":"Rambla.Handlers.Amqp.html#start_link/1"},{"type":"module","title":"Rambla.Handlers.Clickhouse","doc":"Default handler for _Clickhouse_ connections. For this handler to work properly,\n  one must configure it with \n\n```elixir\nconfig :rambla, :clickhouse,\n  connections: [\n    conn_1: \"http://default:password@host-master-1:8123/database\"\n  ],\n  channels: [\n    chan_1: [connection: :conn_1]\n  ]\n\n# Then you can access the connection/channel via `Rambla.Handlers.Clickhouse` as\n\nRambla.Handlers.Clickhouse.publish(:chan_1, %{message: %{foo: 42}, table: :events, serializer: Jason})\n```\n\n---\n\nTo install `Clickhouse`, visit https://clickhouse.com/docs\n\n```sql\nCREATE TABLE events\n(\n  source_id UInt32,\n  timestamp DateTime,\n  message String\n)\nENGINE = MergeTree\nPRIMARY KEY (source_id, timestamp)\n```","ref":"Rambla.Handlers.Clickhouse.html"},{"type":"function","title":"Rambla.Handlers.Clickhouse.children_specs/1","doc":"The list of [`child_spec`](https://hexdocs.pm/elixir/Supervisor.html#t:child_spec/0) returned\n  to be embedded into a supervision tree.\n\nKnown options:\n\n- `connection_options` — a `keyword()` or a function of arity one, which is to receive\n  channel names and return connection options as a list\n- `count` — the number of workers in the pool\n- `child_opts` — the options to be passed to the worker’s spec (you won’t need those)","ref":"Rambla.Handlers.Clickhouse.html#children_specs/1"},{"type":"function","title":"Example - Rambla.Handlers.Clickhouse.children_specs/1","doc":"```elixir\nRambla.Handlers.Redis.children_specs(\n  connection_options: [exchange: \"amq.direct\"], count: 3)\n```","ref":"Rambla.Handlers.Clickhouse.html#children_specs/1-example"},{"type":"function","title":"Rambla.Handlers.Clickhouse.extract_options/2","doc":"","ref":"Rambla.Handlers.Clickhouse.html#extract_options/2"},{"type":"function","title":"Rambla.Handlers.Clickhouse.publish/3","doc":"An interface to publish messages using the FSM pool.\n\nThe `id` is the specific to an implementation,\n  for `Amqp` it’d be the channel name, for instance.\n\nThe second parameter would be a payload, or, if the backend supports it,\n  the function of arity one, which would receive back the connection `pid`.","ref":"Rambla.Handlers.Clickhouse.html#publish/3"},{"type":"function","title":"Example - Rambla.Handlers.Clickhouse.publish/3","doc":"```elixir\nRambla.Handlers.Amqp.publish :channel_name, %{foo: :bar}\n```","ref":"Rambla.Handlers.Clickhouse.html#publish/3-example"},{"type":"function","title":"Rambla.Handlers.Clickhouse.start_link/1","doc":"The entry point: this would start a supervisor with all the pools and stuff","ref":"Rambla.Handlers.Clickhouse.html#start_link/1"},{"type":"module","title":"Rambla.Handlers.Httpc","doc":"Default handler for _HTTP_ connections. For this handler to work properly,\n  one must configure it with \n\n```elixir\nconfig :rambla, :httpc,\n  connections: [\n    httpbin: \"https://httpbin.org/post\",\n    remote_conn: [scheme: \"https\", host: \"httpbin.org\", query: \"post\"]\n  ],\n  channels: [\n    chan_1: [connection: :httpbin, options: [headers: [{\"accept\", \"application/json\"}]]]\n  ]\n\n# Then you can access the connection/channel via `Rambla.Handlers.Amqp` as\n\nRambla.Handlers.Httpc.publish(:chan_1, %{message: %{foo: 42}, serializer: Jason})\n```\n\nThe `publish/2` function receives options which might be used to amend anything,\n  including but not limited to `URI`, which would be updated as shown below\n\n```elixir\n  uri =\n    uri\n    |> then(&if(uri_path, do: URI.append_path(&1, uri_path), else: &1))\n    |> then(&if(uri_query, do: URI.append_query(&1, uri_query), else: &1))\n    |> then(&if(uri_merge, do: URI.merge(&1, uri_merge), else: &1))\n```","ref":"Rambla.Handlers.Httpc.html"},{"type":"function","title":"Rambla.Handlers.Httpc.children_specs/1","doc":"The list of [`child_spec`](https://hexdocs.pm/elixir/Supervisor.html#t:child_spec/0) returned\n  to be embedded into a supervision tree.\n\nKnown options:\n\n- `connection_options` — a `keyword()` or a function of arity one, which is to receive\n  channel names and return connection options as a list\n- `count` — the number of workers in the pool\n- `child_opts` — the options to be passed to the worker’s spec (you won’t need those)","ref":"Rambla.Handlers.Httpc.html#children_specs/1"},{"type":"function","title":"Example - Rambla.Handlers.Httpc.children_specs/1","doc":"```elixir\nRambla.Handlers.Redis.children_specs(\n  connection_options: [exchange: \"amq.direct\"], count: 3)\n```","ref":"Rambla.Handlers.Httpc.html#children_specs/1-example"},{"type":"function","title":"Rambla.Handlers.Httpc.do_handle_publish/3","doc":"","ref":"Rambla.Handlers.Httpc.html#do_handle_publish/3"},{"type":"function","title":"Rambla.Handlers.Httpc.extract_options/2","doc":"","ref":"Rambla.Handlers.Httpc.html#extract_options/2"},{"type":"function","title":"Rambla.Handlers.Httpc.publish/3","doc":"An interface to publish messages using the FSM pool.\n\nThe `id` is the specific to an implementation,\n  for `Amqp` it’d be the channel name, for instance.\n\nThe second parameter would be a payload, or, if the backend supports it,\n  the function of arity one, which would receive back the connection `pid`.","ref":"Rambla.Handlers.Httpc.html#publish/3"},{"type":"function","title":"Example - Rambla.Handlers.Httpc.publish/3","doc":"```elixir\nRambla.Handlers.Amqp.publish :channel_name, %{foo: :bar}\n```","ref":"Rambla.Handlers.Httpc.html#publish/3-example"},{"type":"function","title":"Rambla.Handlers.Httpc.start_link/1","doc":"The entry point: this would start a supervisor with all the pools and stuff","ref":"Rambla.Handlers.Httpc.html#start_link/1"},{"type":"module","title":"Rambla.Handlers.Redis","doc":"Default handler for _Redis_ connections. For this handler to work properly,\n  one must configure it with \n\n```elixir\nconfig :rambla, :redis,\n  connections: [\n    local_conn: \"redis://localhost:6379/0\",\n    remote_conn: [host: \"example.com\", port: 6379, database: 0]\n  ],\n  channels: [\n    chan_1: [connection: :local_conn]\n  ]\n\n# Then you can access the connection/channel via `Rambla.Handlers.Amqp` as\n\nRambla.Handlers.Redis.publish(:chan_1, %{message: %{foo: 42}, serializer: Jason})\n```","ref":"Rambla.Handlers.Redis.html"},{"type":"function","title":"Rambla.Handlers.Redis.children_specs/1","doc":"The list of [`child_spec`](https://hexdocs.pm/elixir/Supervisor.html#t:child_spec/0) returned\n  to be embedded into a supervision tree.\n\nKnown options:\n\n- `connection_options` — a `keyword()` or a function of arity one, which is to receive\n  channel names and return connection options as a list\n- `count` — the number of workers in the pool\n- `child_opts` — the options to be passed to the worker’s spec (you won’t need those)","ref":"Rambla.Handlers.Redis.html#children_specs/1"},{"type":"function","title":"Example - Rambla.Handlers.Redis.children_specs/1","doc":"```elixir\nRambla.Handlers.Redis.children_specs(\n  connection_options: [exchange: \"amq.direct\"], count: 3)\n```","ref":"Rambla.Handlers.Redis.html#children_specs/1-example"},{"type":"function","title":"Rambla.Handlers.Redis.extract_options/2","doc":"","ref":"Rambla.Handlers.Redis.html#extract_options/2"},{"type":"function","title":"Rambla.Handlers.Redis.publish/3","doc":"An interface to publish messages using the FSM pool.\n\nThe `id` is the specific to an implementation,\n  for `Amqp` it’d be the channel name, for instance.\n\nThe second parameter would be a payload, or, if the backend supports it,\n  the function of arity one, which would receive back the connection `pid`.","ref":"Rambla.Handlers.Redis.html#publish/3"},{"type":"function","title":"Example - Rambla.Handlers.Redis.publish/3","doc":"```elixir\nRambla.Handlers.Amqp.publish :channel_name, %{foo: :bar}\n```","ref":"Rambla.Handlers.Redis.html#publish/3-example"},{"type":"function","title":"Rambla.Handlers.Redis.start_link/1","doc":"The entry point: this would start a supervisor with all the pools and stuff","ref":"Rambla.Handlers.Redis.html#start_link/1"},{"type":"module","title":"Rambla.Handlers.S3","doc":"Default handler for _S3 connections. For this handler to work properly,\n  one must configure it with \n\n```elixir\n# config :ex_aws,\n#   access_key_id: [{:system, \"AWS_ACCESS_KEY_ID\"}, {:awscli, \"default\", 30}, :instance_role],\n#   secret_access_key: [{:system, \"AWS_SECRET_ACCESS_KEY\"}, {:awscli, \"default\", 30}, :instance_role]\n\nconfig :ex_aws, \n  access_key_id: [{:system, \"AWS_ACCESS_KEY_ID\"}, :instance_role],\n  secret_access_key: [{:system, \"AWS_SECRET_ACCESS_KEY\"}, :instance_role],\n  hackney_opts: [follow_redirect: true, recv_timeout: 30_000],\n  region: {:system, \"AWS_REGION\"},\n  json_codec: Jason,\n  normalize_path: false,\n  retries: [\n    max_attempts: 1,\n    base_backoff_in_ms: 10,\n    max_backoff_in_ms: 10_000\n  ]\n\nconfig :rambla, :s3,\n  connections: [\n    bucket_1: [bucket: \"test-bucket\", path: \"\"]\n  ],\n  channels: [\n    chan_1: [\n      connection: :bucket_1,\n      options: [connector: ExAws]\n    ]\n  ]\n\n# Then you can access the connection/channel via `Rambla.Handlers.S3` as\n\nRambla.Handlers.S3.publish(:chan_1, %{message: \"Hi John!\", connector: Rambla.Mocks.ExAws})\n```","ref":"Rambla.Handlers.S3.html"},{"type":"function","title":"Rambla.Handlers.S3.children_specs/1","doc":"The list of [`child_spec`](https://hexdocs.pm/elixir/Supervisor.html#t:child_spec/0) returned\n  to be embedded into a supervision tree.\n\nKnown options:\n\n- `connection_options` — a `keyword()` or a function of arity one, which is to receive\n  channel names and return connection options as a list\n- `count` — the number of workers in the pool\n- `child_opts` — the options to be passed to the worker’s spec (you won’t need those)","ref":"Rambla.Handlers.S3.html#children_specs/1"},{"type":"function","title":"Example - Rambla.Handlers.S3.children_specs/1","doc":"```elixir\nRambla.Handlers.Redis.children_specs(\n  connection_options: [exchange: \"amq.direct\"], count: 3)\n```","ref":"Rambla.Handlers.S3.html#children_specs/1-example"},{"type":"function","title":"Rambla.Handlers.S3.do_handle_publish/5","doc":"","ref":"Rambla.Handlers.S3.html#do_handle_publish/5"},{"type":"function","title":"Rambla.Handlers.S3.extract_options/2","doc":"","ref":"Rambla.Handlers.S3.html#extract_options/2"},{"type":"function","title":"Rambla.Handlers.S3.publish/3","doc":"An interface to publish messages using the FSM pool.\n\nThe `id` is the specific to an implementation,\n  for `Amqp` it’d be the channel name, for instance.\n\nThe second parameter would be a payload, or, if the backend supports it,\n  the function of arity one, which would receive back the connection `pid`.","ref":"Rambla.Handlers.S3.html#publish/3"},{"type":"function","title":"Example - Rambla.Handlers.S3.publish/3","doc":"```elixir\nRambla.Handlers.Amqp.publish :channel_name, %{foo: :bar}\n```","ref":"Rambla.Handlers.S3.html#publish/3-example"},{"type":"function","title":"Rambla.Handlers.S3.start_link/1","doc":"The entry point: this would start a supervisor with all the pools and stuff","ref":"Rambla.Handlers.S3.html#start_link/1"},{"type":"module","title":"Rambla.Handlers.Smtp","doc":"Default handler for _SMTP_ connections. For this handler to work properly,\n  one must configure it with \n\n```elixir\nconfig :rambla, :smtp,\n  connections: [\n    gmail: [\n      # the smtp relay, e.g. \"smtp.gmail.com\"\n      relay: System.get_env(\"RAMBLA_SMTP_RELAY\", \"smtp.gmail.com\"),\n      # the username of the smtp relay e.g. \"me@gmail.com\"\n      username: System.get_env(\"RAMBLA_SMTP_USERNAME\"),\n      # the password of the smtp relay e.g. \"mypassword\"\n      password: System.get_env(\"RAMBLA_SMTP_PASSWORD\"),\n      # whether the smtp server needs authentication, valid values are if_available and always,\n      #   Defaults to if_available. If your smtp relay requires authentication set it to always\n      auth: :always,\n      # whether to connect on 465 in ssl mode, Defaults to false\n      ssl: true,\n      # valid values are always, never, if_available.\n      #   Most modern smtp relays use tls, so set this to always, Defaults to if_available\n      # tls: :always,\n      # used in ssl:connect, More info at http://erlang.org/doc/man/ssl.html ,\n      #   Defaults to [{versions , ['tlsv1', 'tlsv1.1', 'tlsv1.2']}],\n      #   This is merged with options listed at: https://github.com/gen-smtp/gen_smtp/blob/master/src/smtp_socket.erl#L46 .\n      #   Any options not present in this list will be ignored.\n      # tls_options: [versions: [:tlsv1, :\"tlsv1.1\", :\"tlsv1.2\"]],\n      # the hostname to be used by the smtp relay,\n      #   Defaults to: smtp_util:guess_FQDN().\n      #   The hostname on your computer might not be correct, so set this to a valid value.\n      hostname: System.get_env(\"RAMBLA_SMTP_HOSTNAME\", \"gmail.com\"),\n      from: %{\"Aleksei Matiushkin\" => \"matiouchkine@gmail.com\"}\n    ]\n  ],\n  channels: [\n    chan_3: [connection: :gmail]\n  ]\n\n# Then you can access the connection/channel via `Rambla.Handlers.Smtp` as\n\nRambla.Handlers.Smtp.publish(:chan_3, \"Hi John!\\nHow are you?\")\n```","ref":"Rambla.Handlers.Smtp.html"},{"type":"function","title":"Rambla.Handlers.Smtp.children_specs/1","doc":"The list of [`child_spec`](https://hexdocs.pm/elixir/Supervisor.html#t:child_spec/0) returned\n  to be embedded into a supervision tree.\n\nKnown options:\n\n- `connection_options` — a `keyword()` or a function of arity one, which is to receive\n  channel names and return connection options as a list\n- `count` — the number of workers in the pool\n- `child_opts` — the options to be passed to the worker’s spec (you won’t need those)","ref":"Rambla.Handlers.Smtp.html#children_specs/1"},{"type":"function","title":"Example - Rambla.Handlers.Smtp.children_specs/1","doc":"```elixir\nRambla.Handlers.Redis.children_specs(\n  connection_options: [exchange: \"amq.direct\"], count: 3)\n```","ref":"Rambla.Handlers.Smtp.html#children_specs/1-example"},{"type":"function","title":"Rambla.Handlers.Smtp.do_handle_publish/3","doc":"","ref":"Rambla.Handlers.Smtp.html#do_handle_publish/3"},{"type":"function","title":"Rambla.Handlers.Smtp.extract_options/2","doc":"","ref":"Rambla.Handlers.Smtp.html#extract_options/2"},{"type":"function","title":"Rambla.Handlers.Smtp.publish/3","doc":"An interface to publish messages using the FSM pool.\n\nThe `id` is the specific to an implementation,\n  for `Amqp` it’d be the channel name, for instance.\n\nThe second parameter would be a payload, or, if the backend supports it,\n  the function of arity one, which would receive back the connection `pid`.","ref":"Rambla.Handlers.Smtp.html#publish/3"},{"type":"function","title":"Example - Rambla.Handlers.Smtp.publish/3","doc":"```elixir\nRambla.Handlers.Amqp.publish :channel_name, %{foo: :bar}\n```","ref":"Rambla.Handlers.Smtp.html#publish/3-example"},{"type":"function","title":"Rambla.Handlers.Smtp.start_link/1","doc":"The entry point: this would start a supervisor with all the pools and stuff","ref":"Rambla.Handlers.Smtp.html#start_link/1"},{"type":"behaviour","title":"Rambla.Handlers.Mock","doc":"Default handler for _Mock_ testing doubles. \n\nNormally, the `test.exs` config would have included all the channels\n  under `config :rambla, mock: […]` key. This would allow testing actual\n  interactions using `Mox` library.\n\nBy default it’d be simply send the message back to the caller.\n\n```elixir\nconfig :rambla, mock: [\n  connections: [mocked: :conn],\n  channels: [chan_0: [connection: :mocked]]\n]\n\n# Then you can access the connection/channel explicitly via `Rambla.Handlers.Mock`\n#   or implicitly via `Rambla` as\n\nRambla.publish(:chan_0, %{message: %{foo: 42}, serializer: Jason})\nRambla.Handlers.Mock.publish(:chan_0, %{message: %{foo: 42}, serializer: Jason})\n```","ref":"Rambla.Handlers.Mock.html"},{"type":"function","title":"Rambla.Handlers.Mock.children_specs/1","doc":"The list of [`child_spec`](https://hexdocs.pm/elixir/Supervisor.html#t:child_spec/0) returned\n  to be embedded into a supervision tree.\n\nKnown options:\n\n- `connection_options` — a `keyword()` or a function of arity one, which is to receive\n  channel names and return connection options as a list\n- `count` — the number of workers in the pool\n- `child_opts` — the options to be passed to the worker’s spec (you won’t need those)","ref":"Rambla.Handlers.Mock.html#children_specs/1"},{"type":"function","title":"Example - Rambla.Handlers.Mock.children_specs/1","doc":"```elixir\nRambla.Handlers.Redis.children_specs(\n  connection_options: [exchange: \"amq.direct\"], count: 3)\n```","ref":"Rambla.Handlers.Mock.html#children_specs/1-example"},{"type":"function","title":"Rambla.Handlers.Mock.do_handle_publish/4","doc":"","ref":"Rambla.Handlers.Mock.html#do_handle_publish/4"},{"type":"function","title":"Rambla.Handlers.Mock.extract_options/2","doc":"","ref":"Rambla.Handlers.Mock.html#extract_options/2"},{"type":"callback","title":"Rambla.Handlers.Mock.on_publish/3","doc":"","ref":"Rambla.Handlers.Mock.html#c:on_publish/3"},{"type":"function","title":"Rambla.Handlers.Mock.publish/3","doc":"An interface to publish messages using the FSM pool.\n\nThe `id` is the specific to an implementation,\n  for `Amqp` it’d be the channel name, for instance.\n\nThe second parameter would be a payload, or, if the backend supports it,\n  the function of arity one, which would receive back the connection `pid`.","ref":"Rambla.Handlers.Mock.html#publish/3"},{"type":"function","title":"Example - Rambla.Handlers.Mock.publish/3","doc":"```elixir\nRambla.Handlers.Amqp.publish :channel_name, %{foo: :bar}\n```","ref":"Rambla.Handlers.Mock.html#publish/3-example"},{"type":"function","title":"Rambla.Handlers.Mock.start_link/1","doc":"The entry point: this would start a supervisor with all the pools and stuff","ref":"Rambla.Handlers.Mock.html#start_link/1"},{"type":"behaviour","title":"Rambla.Handlers.Stub","doc":"Default handler for _Stub_ testing doubles. Unlike `Rambla.Handlers.Mock`,\n  this module might be used as a stub for remote service calls when \n  no expectation is to be defined, or when there is no room to define\n  such an expectation (e. g. while application start.)\n\nBy default it’d be simply return `:ok`.\n\n```elixir\nconfig :rambla, stub: [\n  connections: [stubbed: :conn],\n  channels: [chan_0: [connection: :stubbed]]\n]\n\n# Then you can access the connection/channel via `Rambla.Handlers.Stub` or\n#   implicitly via `Rambla` as\n\nRambla.Handlers.Stub.publish(:chan_0, %{message: %{foo: 42}, serializer: Jason})\nRambla.publish(:chan_0, %{message: %{foo: 42}, serializer: Jason})\n```","ref":"Rambla.Handlers.Stub.html"},{"type":"behaviour","title":"Stub modules - Rambla.Handlers.Stub","doc":"To implement the custom `Stub`, returning any value, or like, use\n\n```elixir\ndefmodule ConnStub do\n  use Rambla.Handlers.Stub, %{token: \"FOOBAR\"}\n\n  @behaviour Rambla.Handlers.Stub\n  def on_publish(_name, _message, _options) do\n    {:ok, @stub_options}\n  end\nend\n```","ref":"Rambla.Handlers.Stub.html#module-stub-modules"},{"type":"function","title":"Rambla.Handlers.Stub.children_specs/1","doc":"The list of [`child_spec`](https://hexdocs.pm/elixir/Supervisor.html#t:child_spec/0) returned\n  to be embedded into a supervision tree.\n\nKnown options:\n\n- `connection_options` — a `keyword()` or a function of arity one, which is to receive\n  channel names and return connection options as a list\n- `count` — the number of workers in the pool\n- `child_opts` — the options to be passed to the worker’s spec (you won’t need those)","ref":"Rambla.Handlers.Stub.html#children_specs/1"},{"type":"function","title":"Example - Rambla.Handlers.Stub.children_specs/1","doc":"```elixir\nRambla.Handlers.Redis.children_specs(\n  connection_options: [exchange: \"amq.direct\"], count: 3)\n```","ref":"Rambla.Handlers.Stub.html#children_specs/1-example"},{"type":"function","title":"Rambla.Handlers.Stub.do_handle_publish/4","doc":"","ref":"Rambla.Handlers.Stub.html#do_handle_publish/4"},{"type":"function","title":"Rambla.Handlers.Stub.extract_options/2","doc":"","ref":"Rambla.Handlers.Stub.html#extract_options/2"},{"type":"callback","title":"Rambla.Handlers.Stub.on_publish/3","doc":"","ref":"Rambla.Handlers.Stub.html#c:on_publish/3"},{"type":"function","title":"Rambla.Handlers.Stub.publish/3","doc":"An interface to publish messages using the FSM pool.\n\nThe `id` is the specific to an implementation,\n  for `Amqp` it’d be the channel name, for instance.\n\nThe second parameter would be a payload, or, if the backend supports it,\n  the function of arity one, which would receive back the connection `pid`.","ref":"Rambla.Handlers.Stub.html#publish/3"},{"type":"function","title":"Example - Rambla.Handlers.Stub.publish/3","doc":"```elixir\nRambla.Handlers.Amqp.publish :channel_name, %{foo: :bar}\n```","ref":"Rambla.Handlers.Stub.html#publish/3-example"},{"type":"function","title":"Rambla.Handlers.Stub.start_link/1","doc":"The entry point: this would start a supervisor with all the pools and stuff","ref":"Rambla.Handlers.Stub.html#start_link/1"},{"type":"behaviour","title":"Rambla.Connection","doc":"The default behaviour for publishers. The common use case would be the module\nimplementing this behaviour opens a connection (and keep it opened,) and\npublishes messages as needed.\n\n---\n\n`Rambla.Connection` supports `options:` config that will be passed directly to\n`:poolboy` as [`Options`](https://github.com/devinus/poolboy#options).","ref":"Rambla.Connection.html"},{"type":"behaviour","title":"Example - Rambla.Connection","doc":"```elixir\nconfig :rambla,\n  amqp: […]\n  pools: [amqp: [options: [size: 2, max_overflow: 1], …]]\n```","ref":"Rambla.Connection.html#module-example"},{"type":"function","title":"Rambla.Connection.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Rambla.Connection.html#child_spec/1"},{"type":"callback","title":"Rambla.Connection.connect/1","doc":"Connects to the remote service and returns a connection object back","ref":"Rambla.Connection.html#c:connect/1"},{"type":"type","title":"Rambla.Connection.message/0","doc":"The accepted type of the message to be published","ref":"Rambla.Connection.html#t:message/0"},{"type":"type","title":"Rambla.Connection.messages/0","doc":"","ref":"Rambla.Connection.html#t:messages/0"},{"type":"type","title":"Rambla.Connection.outcome/0","doc":"The response type for the single request.\nContains a status and a response from remote service","ref":"Rambla.Connection.html#t:outcome/0"},{"type":"type","title":"Rambla.Connection.outcomes/0","doc":"The response type for the bulk request.\nContains a status and a response from remote service","ref":"Rambla.Connection.html#t:outcomes/0"},{"type":"callback","title":"Rambla.Connection.publish/2","doc":"Publishes the message to the remote service using connection provided","ref":"Rambla.Connection.html#c:publish/2"},{"type":"function","title":"Rambla.Connection.start_link/1","doc":"Accepts options for the underlying connection (those will be passed to `connect/1`.)","ref":"Rambla.Connection.html#start_link/1"},{"type":"type","title":"Rambla.Connection.t/0","doc":"The connection information","ref":"Rambla.Connection.html#t:t/0"},{"type":"module","title":"Rambla.Connection.Config","doc":"The connection settings as requested by connection provider","ref":"Rambla.Connection.Config.html"},{"type":"type","title":"Rambla.Connection.Config.t/0","doc":"The configuration of the real connection behind a pool","ref":"Rambla.Connection.Config.html#t:t/0"},{"type":"module","title":"Rambla.Amqp","doc":"Default connection implementation for 🐰 Rabbit.\n\n`publish/2` accepts the following options:\n\n- `exchange` [`binary()`, **mandatory**] the exchange to publish to\n- `queue` [`binary()`, **optional**] if passed, the queue will be created\n  and bound to the exchange; it’s slowing down publishing, but safer\n  for the cold RebbatMQ installation\n- `declare?`[`boolean()`, **optional**, _default_: `true`] if false\n  is passed, the exchange would not be declared; use it if the exchange\n  already surely exists to speed up the publishing\n- `routing_key` [`binary()`, **optional**, _default_: `\"\"`] if passed,\n  used as a routing key\n- `options` [`keyword()`, **optional**, _default_: `[]`] the options\n  to be passed as is to call to `AMQP.Basic.publish/5`\n\n---\n\nSince `v0.6.0` provides two `mix` tasks:\n\n- `mix rambla.rabbit.exchange` Operations with exchanges in RabbitMQ\n- `mix rambla.rabbit.queue`    Operations with queues in RabbitMQ\n\nTasks support arguments to be passed to RabbitMQ instance. Usage example:\n\n```\nmix rambla.rabbit.queue declare foo -o durable:true\n```","ref":"Rambla.Amqp.html"},{"type":"module","title":"Rambla.Http","doc":"Default connection implementation for 🕸️ HTTP.\n\nIt expects a message to be a map, containing the following fields:\n`:method`, `:path`, `:query`, `:body` _and_ the optional `:type`\nthat otherwise would be inferred from the body type.\n\nFor instance, this call would send a POST request with a JSON specified as body.\n\n```elixir\nRambla.publish(\n  Rambla.Http,\n  %{method: :post, body: %{message: \"I ❤ HTTP\"}}\n}\n```\n\nIf the second argument `message` is `binary()` it’s treated as an URL _and_\n`:get` is implied.\n\n---\n\nList of all possible options might be found in\n[`:httpc.request/4`](http://erlang.org/doc/man/httpc.html#request-4), names are preserved.","ref":"Rambla.Http.html"},{"type":"module","title":"Rambla.Process","doc":"Default connection implementation for 🕸️ Process message callback.\n\nFor instance, this call would send a message back to the calling process.\n\n```elixir\nRambla.publish(\n  Rambla.Process,\n  %{method: :post, body: %{message: \"I ❤ HTTP\"}}\n}\n```","ref":"Rambla.Process.html"},{"type":"module","title":"Rambla.Redis","doc":"Default connection implementation for 🔴 Redis.","ref":"Rambla.Redis.html"},{"type":"module","title":"Rambla.Smtp","doc":"Default connection implementation for 📧 SMTP.\n\nIt expects a message to be a map containing the following fields:\n`:to`, `:subject`, `:body` _and_ the optional `:from` that otherwise would be\ntaken from the global settings (`releases.mix`) from `[]:rambla, :pools, Rambla.Smtp]`.\n\nFor instance, this call would send an email to email:am@example.com with the\nrespective subject and body.\n\n```elixir\nRambla.publish(\n  Rambla.Smtp,\n  %{to: \"am@example.com\", subject: \"Hi there\", body: \"I ❤ SMTP\"}\n}\n```","ref":"Rambla.Smtp.html"},{"type":"module","title":"Rambla.Exception","doc":"Base type for all the `Rambla` exceptions.","ref":"Rambla.Exception.html"},{"type":"type","title":"Rambla.Exception.t/0","doc":"`Rambla` exception contains:\n\n- `reason` the string describing the error in human-readable form\n- `source` usually the type/action this error is originated from\n- `info` free-style map containig additional information (not displayed by default)","ref":"Rambla.Exception.html#t:t/0"},{"type":"extras","title":"Rambla","doc":"# Rambla    [![Kantox ❤ OSS](https://img.shields.io/badge/❤-kantox_oss-informational.svg)](https://kantox.com/)  ![Test](https://github.com/am-kantox/rambla/workflows/Test/badge.svg)  ![Dialyzer](https://github.com/am-kantox/rambla/workflows/Dialyzer/badge.svg)\n\n**Easy publishing to many different targets**","ref":"readme.html"},{"type":"extras","title":"Installation - Rambla","doc":"```elixir\ndef deps do\n  [\n    {:rambla, \"~> 0.4\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Supported backends - Rambla","doc":"- Rabbit (through [Amqp](https://hexdocs.pm/amqp/))\n- Redis (through [Redix](https://hexdocs.pm/redix))\n- Http (through [:httpc](http://erlang.org/doc/man/httpc.html))\n- Smtp (through [:gen_smtp](https://hexdocs.pm/gen_smtp))\n- S3 (through [:ex_aws_s3](https://hexdocs.pm/ex_aws_s3))\n- Slack (through [Envío](https://hexdocs.pm/envio)) [pending]","ref":"readme.html#supported-backends"},{"type":"extras","title":"Coming soon - Rambla","doc":"- AWS","ref":"readme.html#coming-soon"},{"type":"extras","title":"Changelog - Rambla","doc":"- **`1.2.6`** Allow `runtime.exs` config for services\n- **`1.2.3`** Allow patching `URI` in `:httpc` backend\n- **`1.2.1`** `Clickhouse` backend\n- **`1.2.0`** allow `on_success/1` and `on_failure/1` callbacks to better control the execution \n- **`1.1.5`** `preferred_format: :map | :binary | :none`\n- **`1.1.4`** `use Rambla.Handlers.Stub`\n- **`1.1.3`** `Rambla.Handlers.Stub`\n- **`1.1.1`** `Rambla.Handlers.Mock`\n- **`1.1.0`** Complete rewrite, all the services are now controlled by `Finitomata.Pool`\n- **`1.0.0`** Modern era update\n- **`0.16.3`** Better error reporting, logging of insuccessfuly HTTP requests\n- **`0.16.0`** Several instances of service + Mocks\n- **`0.15.0`** Hashring for selecting channels in AMQP backend within a single connection\n- **`0.14.4`** Accept headers for `:httpc` as map/keyword of binaries, `:gen_server_timeout` for `publish_synch/3`\n- **`0.14.0`** Use Tarearbol.Pool to manage channels behind AMQP connections\n- **`0.13.0`** Filter out connection params from logs\n- **`0.12.0`** `Rambla.publish_synch/3` to avoid pool while publishing\n- **`0.11.1`** Optional Boundary support for Telemetria\n- **`0.11.0`** Envío → Telemetria\n- **`0.9.3`** Envío broadcast to `:rambla` channel, with a type\n- **`0.9.0`** Divorce `Rambla` with `AMQP` and `Envio`\n- **`0.8.0`** `Rambla.raw/2` returning a worker from pool\n- **`0.6.5`** `RabbitMQ` → `bind`, `unbind`\n- **`0.6.3`** Auto-reenable tasks\n- **`0.6.2`** code cleanup, DRY\n- **`0.6.0`** `mix` tasks to deal with RabbitMQ\n- **`0.5.2`** graceful timeout, fix for optional `Envio` does not included\n- **`0.5.1`** performance fixes, do not require `queue` in call to Rabbit `publish/2`, `declare?: false` to not declare exchange every time\n- **`0.5.0`** bulk publisher\n- **`0.4.0`** `SMTP` publisher\n- **`0.3.0`** `HTTP` publisher","ref":"readme.html#changelog"},{"type":"extras","title":"Documentation - Rambla","doc":"- [https://hexdocs.pm/rambla](https://hexdocs.pm/rambla).","ref":"readme.html#documentation"},{"type":"extras","title":"Getting Started","doc":"# Getting Started","ref":"getting-started.html"},{"type":"extras","title":"Intro - Getting Started","doc":"`Rambla` provides the ability to publish messages to several different destinations. Destinations are supported via handlers. The respective handler must be explicitly included into the list of extra applications of _target application_.\n\nFor each of the configured destinations, the pool of workers based on `Finitomata.Pool` is maintained. Each destination should be configured separately; the preferred way would be to use `config.exs` because the handlers themselves are included in the release based on this config. Additional configuration might be passed to the `Rambla.start_link/1` call.","ref":"getting-started.html#intro"},{"type":"extras","title":"Configuration - Getting Started","doc":"The configuration is the keyword list with keys specifying handlers and their initialization properties. All the configs are based on the pattern provided by [`:amqp`](https://hexdocs.pm/amqp/AMQP.Application.html#get_channel/1-usage) application.","ref":"getting-started.html#configuration"},{"type":"extras","title":"Starting Pools - Getting Started","doc":"Embed `Rambla` into your supervision tree. The _configured_ handlers will be started supervised.","ref":"getting-started.html#starting-pools"},{"type":"extras","title":"Configuration Example - Getting Started","doc":"`Rambla.Handlers.Amqp` requires `:amqp` application to be configured and started, ditto for `Rambla.Handlers.S3`. Everything else is to be configured as shown below.\n\n```elixir\nconfig :rambla,\n  redis: [\n    connections: [\n      local_conn: [\n        host: System.get_env(\"REDIS_HOST\", \"127.0.0.1\"),\n        port: String.to_integer(System.get_env(\"REDIS_PORT\", \"6379\")),\n        password: System.get_env(\"REDIS_PASSWORD\", \"\"),\n        database: 0\n      ]\n    ],\n    channels: [chan_1: [connection: :local_conn]]\n  ],\n  httpc: [\n    connections: [\n      httpbin_success: [scheme: \"https\", host: \"httpbin.org\", path: \"/post\"],\n      httpbin_error: [scheme: \"https\", host: \"httpbin.org\", path: \"/status/500\"]\n    ],\n    channels: [\n      chan_1: [connection: :httpbin_success, options: [headers: [{\"accept\", \"application/json\"}]]],\n      chan_2: [connection: :httpbin_error, options: [headers: [{\"accept\", \"text/plain\"}]]]\n    ]\n  ],\n```","ref":"getting-started.html#configuration-example"},{"type":"extras","title":"Publishing - Getting Started","doc":"Publishing to the destination is as easy, as calling [`Rambla.html#publish/3`](https://hexdocs.pm/rambla/Rambla.html#publish/3) passing the destination channel, the message and optional configuration parameters. The message will be published to all the configured handlers for this channel.\n\nThe following would publish the message to previously configured `:channel_1` channel.\n\n```elixir\nRambla.publish(:chan_1, %{message: %{foo: 42, bar: :baz}, exchange: \"barfoo\"})\n```","ref":"getting-started.html#publishing"},{"type":"extras","title":"Testing - Getting Started","doc":"In `:test` environment, use `Rambla.Handlers.Mock` and `Rambla.Handlers.Stub` handlers to substitute actual destinations for the channels and additionally use `Mox` expectations with `Mock` handler to test the publishing.\n\n```elixir\nconfig :rambla,\n  mock: [\n    connections: [mocked: :conn_mocked],\n    channels: [chan_1: [connection: :mocked]]\n  ],\n  stub: [\n    connections: [stubbed: :conn_stubbed],\n    channels: [chan_2: [connection: :stubbed]]\n  ]\n```","ref":"getting-started.html#testing"},{"type":"extras","title":"Testing Rambla","doc":"# Testing Rambla\n\nThere are two ways to test `Rambla` in the wild. One might do a granular testing as shown\nbelow with `S3` mock example.\n\n```elixir\nexpect(Rambla.Mocks.ExAws, :request, fn operation, %{} = _params ->\n  assert %ExAws.Operation.S3{} = operation\n  assert operation.http_method == :put\n  assert operation.bucket == \"test-bucket\"\n  assert operation.path == \"some/path\"\n\n  {:ok, %{body: \"file contents\"}}\nend)\n\nRambla.Handlers.S3.publish(:chan_1, %{message: \"file contents\"}, self())\nassert_receive {:transition, :success, _, _}, 1_000\n```\n\nAnother options would be to use `Rambla.Handlers.Mock` with a custom mock.\n\n```elixir\nexpect(Rambla.Mocks.Generic, :on_publish, fn name, message, %{} = _opts ->\n  assert name == :chan_0\n  assert message == \"file contents\"\n\n  {:ok, %{body: \"file contents\"}}\nend)\n\nRambla.publish(:chan_0, %{message: \"file contents\"}, self())\nassert_receive {:transition, :success, _, _}, 1_000\n```","ref":"testing.html"},{"type":"extras","title":"Backends","doc":"# Backends\n\nThis document provides detailed information about each supported backend in Rambla, including configuration options, usage examples, and common patterns.","ref":"backends.html"},{"type":"extras","title":"Documentation Index - Backends","doc":"- [README](../README.md) - Overview and Quick Start\n- [Backend Documentation](backends.md) - Detailed backend guide\n- [Custom Backends](custom_backends.md) - Creating custom backends\n- [Configuration Reference](configuration.md) - Complete configuration guide","ref":"backends.html#documentation-index"},{"type":"extras","title":"Table of Contents - Backends","doc":"- [AMQP (RabbitMQ)](#amqp-rabbitmq)\n- [Redis](#redis)\n- [HTTP](#http)\n- [SMTP](#smtp)\n- [S3](#s3)\n- [ClickHouse](#clickhouse)\n- [Common Options](#common-options)","ref":"backends.html#table-of-contents"},{"type":"extras","title":"AMQP (RabbitMQ) - Backends","doc":"The AMQP backend provides integration with RabbitMQ through the `amqp` library.","ref":"backends.html#amqp-rabbitmq"},{"type":"extras","title":"Configuration - Backends","doc":"```elixir\nconfig :rambla,\n  amqp: [\n    connections: [\n      local_conn: [\n        url: \"amqp://guest:guest@localhost:5672\",\n        # or detailed configuration:\n        # host: \"localhost\",\n        # port: 5672,\n        # username: \"guest\",\n        # password: \"guest\",\n        # virtual_host: \"/\"\n      ]\n    ],\n    channels: [\n      chan_1: [\n        connection: :local_conn,\n        options: [\n          declare?: true,         # Whether to declare exchange on publish\n          exchange: \"my-exchange\",\n          routing_key: \"my-route\",\n          callbacks: [\n            on_success: fn result -> Logger.info(\"Published: #{inspect(result)}\") end,\n            on_failure: fn error -> Logger.error(\"Failed: #{inspect(error)}\") end\n          ]\n        ]\n      ]\n    ]\n  ]\n```","ref":"backends.html#configuration"},{"type":"extras","title":"Usage Examples - Backends","doc":"```elixir\n# Basic publish\nRambla.publish(:chan_1, %{foo: 42})\n\n# With specific exchange and routing key\nRambla.publish(:chan_1, %{\n  message: %{foo: 42},\n  exchange: \"specific-exchange\",\n  routing_key: \"specific-route\"\n})\n\n# Declare exchange with type\nRambla.publish(:chan_1, %{\n  message: %{foo: 42},\n  exchange: \"my-topic\",\n  declare?: :topic  # Will declare a topic exchange\n})\n\n# Bulk publishing\nRambla.publish(:chan_1, [\n  %{message: %{foo: 1}, routing_key: \"route1\"},\n  %{message: %{foo: 2}, routing_key: \"route2\"}\n])\n```","ref":"backends.html#usage-examples"},{"type":"extras","title":"Redis - Backends","doc":"The Redis backend uses the `redix` library for Redis communication.","ref":"backends.html#redis"},{"type":"extras","title":"Configuration - Backends","doc":"```elixir\nconfig :rambla,\n  redis: [\n    connections: [\n      local_conn: [\n        host: \"localhost\",\n        port: 6379,\n        password: \"\",  # Optional\n        database: 0,\n        ssl: false     # Optional\n      ]\n    ],\n    channels: [\n      chan_1: [\n        connection: :local_conn,\n        options: [\n          preferred_format: :map,  # :map | :binary | :none\n          serializer: Jason       # Optional, defaults to Jason\n        ]\n      ]\n    ]\n  ]\n```","ref":"backends.html#configuration-1"},{"type":"extras","title":"Usage Examples - Backends","doc":"```elixir\n# Basic key-value publish\nRambla.publish(:chan_1, %{key: \"value\"})\n\n# With specific format and serializer\nRambla.publish(:chan_1, %{\n  message: %{foo: 42},\n  preferred_format: :binary,\n  serializer: Jason\n})\n```","ref":"backends.html#usage-examples-1"},{"type":"extras","title":"HTTP - Backends","doc":"The HTTP backend uses Erlang's built-in `:httpc` module.","ref":"backends.html#http"},{"type":"extras","title":"Configuration - Backends","doc":"```elixir\nconfig :rambla,\n  httpc: [\n    connections: [\n      api_conn: [\n        scheme: \"https\",\n        host: \"api.example.com\",\n        path: \"/endpoint\",\n        port: 443           # Optional\n      ]\n    ],\n    channels: [\n      chan_1: [\n        connection: :api_conn,\n        options: [\n          headers: [\n            {\"content-type\", \"application/json\"},\n            {\"accept\", \"application/json\"}\n          ],\n          method: :post,    # Default is :post\n          timeout: 5000     # Timeout in milliseconds\n        ]\n      ]\n    ]\n  ]\n```","ref":"backends.html#configuration-2"},{"type":"extras","title":"Usage Examples - Backends","doc":"```elixir\n# Basic HTTP post\nRambla.publish(:chan_1, %{foo: 42})\n\n# With custom headers and method\nRambla.publish(:chan_1, %{\n  message: %{foo: 42},\n  headers: [{\"x-custom-header\", \"value\"}],\n  method: :put\n})\n\n# With URI path modification\nRambla.publish(:chan_1, %{\n  message: %{foo: 42},\n  uri_merge: \"/additional/path\"\n})\n```","ref":"backends.html#usage-examples-2"},{"type":"extras","title":"SMTP - Backends","doc":"The SMTP backend uses `gen_smtp` for email delivery.","ref":"backends.html#smtp"},{"type":"extras","title":"Configuration - Backends","doc":"```elixir\nconfig :rambla,\n  smtp: [\n    connections: [\n      gmail: [\n        relay: \"smtp.gmail.com\",\n        port: 587,\n        username: \"your-email@gmail.com\",\n        password: \"your-password\",\n        tls: :always,\n        auth: :always\n      ]\n    ],\n    channels: [\n      chan_1: [\n        connection: :gmail,\n        options: [\n          from: \"sender@example.com\",\n          headers: [\n            {\"Content-Type\", \"text/html; charset=UTF-8\"}\n          ]\n        ]\n      ]\n    ]\n  ]\n```","ref":"backends.html#configuration-3"},{"type":"extras","title":"Usage Examples - Backends","doc":"```elixir\n# Send email\nRambla.publish(:chan_1, %{\n  to: \"recipient@example.com\",\n  subject: \"Test Email\",\n  body: \"Hello from Rambla!\"\n})\n\n# With HTML content and custom headers\nRambla.publish(:chan_1, %{\n  message: %{\n    to: \"recipient@example.com\",\n    subject: \"HTML Test\",\n    body: \" Hello  This is HTML email \"\n  },\n  headers: [\n    {\"Content-Type\", \"text/html; charset=UTF-8\"},\n    {\"X-Priority\", \"1\"}\n  ]\n})\n```","ref":"backends.html#usage-examples-3"},{"type":"extras","title":"S3 - Backends","doc":"The S3 backend uses `ex_aws_s3` for Amazon S3 integration.","ref":"backends.html#s3"},{"type":"extras","title":"Configuration - Backends","doc":"```elixir\nconfig :rambla,\n  s3: [\n    connections: [\n      default: [\n        bucket: \"my-bucket\",\n        region: \"eu-west-1\"\n      ]\n    ],\n    channels: [\n      chan_1: [\n        connection: :default,\n        options: [\n          acl: :private,\n          content_type: \"application/json\"\n        ]\n      ]\n    ]\n  ]\n\n# Additional AWS configuration\nconfig :ex_aws,\n  access_key_id: \"your-access-key\",\n  secret_access_key: \"your-secret-key\"\n```","ref":"backends.html#configuration-4"},{"type":"extras","title":"Usage Examples - Backends","doc":"```elixir\n# Upload file to S3\nRambla.publish(:chan_1, %{\n  message: \"file contents\",\n  path: \"path/to/file.txt\"\n})\n\n# With custom options\nRambla.publish(:chan_1, %{\n  message: Jason.encode!(%{foo: 42}),\n  path: \"data/record.json\",\n  acl: :public_read,\n  content_type: \"application/json\"\n})\n```","ref":"backends.html#usage-examples-4"},{"type":"extras","title":"ClickHouse - Backends","doc":"The ClickHouse backend uses `pillar` for database operations.","ref":"backends.html#clickhouse"},{"type":"extras","title":"Configuration - Backends","doc":"```elixir\nconfig :rambla,\n  clickhouse: [\n    connections: [\n      default: [\n        host: \"localhost\",\n        port: 8123,\n        database: \"default\",\n        username: \"default\",\n        password: \"\"\n      ]\n    ],\n    channels: [\n      chan_1: [\n        connection: :default,\n        options: [\n          table: \"events\",\n          batch_size: 1000  # Optional\n        ]\n      ]\n    ]\n  ]\n```","ref":"backends.html#configuration-5"},{"type":"extras","title":"Usage Examples - Backends","doc":"```elixir\n# Insert single record\nRambla.publish(:chan_1, %{\n  table: :events,\n  message: %{\n    timestamp: DateTime.utc_now(),\n    event_type: \"user_login\",\n    data: %{user_id: 123}\n  }\n})\n\n# Batch insert\nRambla.publish(:chan_1, %{\n  table: :events,\n  message: [\n    %{timestamp: DateTime.utc_now(), event_type: \"event1\"},\n    %{timestamp: DateTime.utc_now(), event_type: \"event2\"}\n  ]\n})\n```","ref":"backends.html#usage-examples-5"},{"type":"extras","title":"Common Options - Backends","doc":"All backends support these common configuration options:","ref":"backends.html#common-options"},{"type":"extras","title":"Channel Options - Backends","doc":"- `callbacks`: Specify success and failure handlers\n  ```elixir\n  callbacks: [\n    on_success: fn result -> Logger.info(\"Success: #{inspect(result)}\") end,\n    on_failure: fn error -> Logger.error(\"Error: #{inspect(error)}\") end\n  ]\n  ```","ref":"backends.html#channel-options"},{"type":"extras","title":"Message Format - Backends","doc":"- `preferred_format`: Control message serialization\n  - `:map` - Automatically decode JSON strings to maps\n  - `:binary` - Automatically encode maps to JSON strings\n  - `:none` - No automatic conversion","ref":"backends.html#message-format"},{"type":"extras","title":"Retry Configuration - Backends","doc":"- `retries`: Number of retry attempts (default: 5)\n  ```elixir\n  config :rambla,\n    max_retries: 3\n  ```","ref":"backends.html#retry-configuration"},{"type":"extras","title":"Pool Configuration - Backends","doc":"- `count`: Number of workers in the connection pool\n  ```elixir\n  config :rambla,\n    redis: [\n      connections: [\n        local_conn: [\n          count: 5  # 5 workers in the pool\n        ]\n      ]\n    ]\n  ```\n\nFor backend-specific options, refer to the respective backend sections above.","ref":"backends.html#pool-configuration"},{"type":"extras","title":"Troubleshooting - Backends","doc":"Common issues and their solutions:","ref":"backends.html#troubleshooting"},{"type":"extras","title":"Connection Issues - Backends","doc":"1. **AMQP Connection Failures**\n   ```elixir\n   {:error, :not_connected}\n   ```\n   - Check RabbitMQ is running\n   - Verify credentials and vhost\n   - Ensure port is accessible\n\n2. **Redis Connection Timeouts**\n   ```elixir\n   {:error, :timeout}\n   ```\n   - Check Redis server status\n   - Verify network connectivity\n   - Adjust timeout settings\n\n3. **HTTP Connection Errors**\n   ```elixir\n   {:error, :econnrefused}\n   ```\n   - Verify endpoint URL\n   - Check firewall settings\n   - Validate SSL certificates","ref":"backends.html#connection-issues"},{"type":"extras","title":"Common Solutions - Backends","doc":"- Increase logging level for debugging\n- Check connection pool settings\n- Verify environment variables\n- Ensure all dependencies are started\n\nFor more details on configuration options, see the [Configuration Guide](configuration.md#error-handling).","ref":"backends.html#common-solutions"},{"type":"extras","title":"Configuration Guide","doc":"# Configuration Guide\n\nThis document provides a comprehensive reference for configuring Rambla, including global settings, backend-specific configurations, and advanced options.","ref":"configuration.html"},{"type":"extras","title":"Documentation Index - Configuration Guide","doc":"- [README](../README.md) - Overview and Quick Start\n- [Backend Documentation](backends.md) - Detailed backend guide\n- [Custom Backends](custom_backends.md) - Creating custom backends\n- [Configuration Reference](configuration.md) - Complete configuration guide","ref":"configuration.html#documentation-index"},{"type":"extras","title":"Table of Contents - Configuration Guide","doc":"- [Basic Configuration](#basic-configuration)\n- [Global Settings](#global-settings)\n- [Runtime Configuration](#runtime-configuration)\n- [Pool Configuration](#pool-configuration)\n- [Environment Variables](#environment-variables)\n- [Advanced Options](#advanced-options)\n- [Best Practices](#best-practices)","ref":"configuration.html#table-of-contents"},{"type":"extras","title":"Basic Configuration - Configuration Guide","doc":"Rambla follows a consistent configuration pattern across all backends:\n\n```elixir\nconfig :rambla,\n  max_retries: 5,    # Global retry setting\n  enable_deprecated: false,  # Disable deprecated functionality\n  \n  # Backend-specific configurations\n  backend_name: [\n    connections: [\n      connection_name: [\n        # Connection parameters\n      ]\n    ],\n    channels: [\n      channel_name: [\n        connection: :connection_name,\n        options: [\n          # Channel-specific options\n        ]\n      ]\n    ]\n  ]\n```","ref":"configuration.html#basic-configuration"},{"type":"extras","title":"Global Settings - Configuration Guide","doc":"Global settings affect all backends:\n\n```elixir\nconfig :rambla,\n  # Maximum retry attempts for failed operations\n  max_retries: 5,\n  \n  # Enable/disable deprecated functionality\n  enable_deprecated: false,\n  \n  # Default serializer for message encoding/decoding\n  serializer: Jason,\n  \n  # Global logging level\n  log_level: :info,\n  \n  # Explicitly list enabled services\n  services: [:amqp, :redis, :http]\n```","ref":"configuration.html#global-settings"},{"type":"extras","title":"Runtime Configuration - Configuration Guide","doc":"Rambla supports runtime configuration through `runtime.exs`:\n\n```elixir\n# config/runtime.exs\nconfig :rambla,\n  redis: [\n    connections: [\n      local_conn: [\n        host: System.get_env(\"REDIS_HOST\", \"localhost\"),\n        port: String.to_integer(System.get_env(\"REDIS_PORT\", \"6379\")),\n        password: System.get_env(\"REDIS_PASSWORD\", \"\")\n      ]\n    ]\n  ]\n```","ref":"configuration.html#runtime-configuration"},{"type":"extras","title":"Pool Configuration - Configuration Guide","doc":"Configure connection pools for each backend:\n\n```elixir\nconfig :rambla,\n  redis: [\n    connections: [\n      local_conn: [\n        # Number of workers in the pool\n        count: 5,\n        \n        # Maximum number of additional workers\n        max_overflow: 10,\n        \n        # Worker startup options\n        worker_options: [\n          timeout: 5000\n        ]\n      ]\n    ]\n  ]\n```","ref":"configuration.html#pool-configuration"},{"type":"extras","title":"Environment Variables - Configuration Guide","doc":"Common environment variables used in configuration:\n\n```elixir\n# AMQP/RabbitMQ\nRABBITMQ_URL=\"amqp://guest:guest@localhost:5672\"\nRABBITMQ_HOST=\"localhost\"\nRABBITMQ_PORT=\"5672\"\nRABBITMQ_USER=\"guest\"\nRABBITMQ_PASSWORD=\"guest\"\nRABBITMQ_VHOST=\"/\"\n\n# Redis\nREDIS_HOST=\"localhost\"\nREDIS_PORT=\"6379\"\nREDIS_PASSWORD=\"\"\nREDIS_DATABASE=\"0\"\n\n# HTTP\nHTTP_BASE_URL=\"https://api.example.com\"\nHTTP_TIMEOUT=\"5000\"\n\n# SMTP\nSMTP_RELAY=\"smtp.gmail.com\"\nSMTP_PORT=\"587\"\nSMTP_USERNAME=\"your-email@gmail.com\"\nSMTP_PASSWORD=\"your-app-password\"\n\n# AWS/S3\nAWS_ACCESS_KEY_ID=\"your-access-key\"\nAWS_SECRET_ACCESS_KEY=\"your-secret-key\"\nAWS_REGION=\"eu-west-1\"\n```","ref":"configuration.html#environment-variables"},{"type":"extras","title":"Advanced Options - Configuration Guide","doc":"","ref":"configuration.html#advanced-options"},{"type":"extras","title":"Channel Callbacks - Configuration Guide","doc":"Configure success and failure handlers:\n\n```elixir\nconfig :rambla,\n  amqp: [\n    channels: [\n      chan_1: [\n        connection: :local_conn,\n        options: [\n          callbacks: [\n            on_success: fn result ->\n              Logger.info(\"Success: #{inspect(result)}\")\n              :ok\n            end,\n            on_failure: fn error ->\n              Logger.warning(\"Failed: #{inspect(error)}\")\n              :retry  # or {:retry, %{retries: 3}}\n            end\n          ]\n        ]\n      ]\n    ]\n  ]\n```","ref":"configuration.html#channel-callbacks"},{"type":"extras","title":"Message Format Control - Configuration Guide","doc":"Configure message serialization:\n\n```elixir\nconfig :rambla,\n  redis: [\n    channels: [\n      chan_1: [\n        connection: :local_conn,\n        options: [\n          # Control message format\n          preferred_format: :map,  # :map | :binary | :none\n          \n          # Custom serializer\n          serializer: Jason,\n          \n          # Serialization options\n          json_opts: [pretty: true]\n        ]\n      ]\n    ]\n  ]\n```","ref":"configuration.html#message-format-control"},{"type":"extras","title":"Error Handling - Configuration Guide","doc":"Configure retry behavior:\n\n```elixir\nconfig :rambla,\n  http: [\n    channels: [\n      chan_1: [\n        connection: :api_conn,\n        options: [\n          # Maximum retry attempts\n          retries: 3,\n          \n          # Backoff strategy\n          backoff: [\n            type: :exponential,\n            initial: 100,\n            multiplier: 2,\n            max: 1000\n          ],\n          \n          # Fatal error handler\n          on_fatal: &MyApp.ErrorHandler.handle_fatal/1\n        ]\n      ]\n    ]\n  ]\n```","ref":"configuration.html#error-handling"},{"type":"extras","title":"Best Practices - Configuration Guide","doc":"","ref":"configuration.html#best-practices"},{"type":"extras","title":"1. Use Environment Variables - Configuration Guide","doc":"Store sensitive information in environment variables:\n\n```elixir\nconfig :rambla,\n  amqp: [\n    connections: [\n      local_conn: [\n        url: {:system, \"RABBITMQ_URL\"},\n        # or detailed configuration\n        host: {:system, \"RABBITMQ_HOST\", \"localhost\"},\n        port: {:system, \"RABBITMQ_PORT\", \"5672\"} |> String.to_integer(),\n        username: {:system, \"RABBITMQ_USER\"},\n        password: {:system, \"RABBITMQ_PASSWORD\"}\n      ]\n    ]\n  ]\n```","ref":"configuration.html#1-use-environment-variables"},{"type":"extras","title":"2. Separate Development and Production Configs - Configuration Guide","doc":"```elixir\n# config/dev.exs\nconfig :rambla,\n  amqp: [\n    connections: [\n      local_conn: [url: \"amqp://guest:guest@localhost:5672\"]\n    ]\n  ]\n\n# config/prod.exs\nconfig :rambla,\n  amqp: [\n    connections: [\n      prod_conn: [\n        url: {:system, \"RABBITMQ_URL\"},\n        ssl_options: [\n          verify: :verify_peer,\n          cacertfile: \"/path/to/ca.crt\"\n        ]\n      ]\n    ]\n  ]\n```","ref":"configuration.html#2-separate-development-and-production-configs"},{"type":"extras","title":"3. Use Channel-Specific Options - Configuration Guide","doc":"Configure options per channel for flexibility:\n\n```elixir\nconfig :rambla,\n  redis: [\n    channels: [\n      # Fast, non-critical channel\n      events_chan: [\n        connection: :local_conn,\n        options: [retries: 1, timeout: 1000]\n      ],\n      \n      # Critical channel with retries\n      orders_chan: [\n        connection: :local_conn,\n        options: [retries: 5, timeout: 5000]\n      ]\n    ]\n  ]\n```","ref":"configuration.html#3-use-channel-specific-options"},{"type":"extras","title":"4. Configure Logging - Configuration Guide","doc":"Use appropriate logging levels:\n\n```elixir\nconfig :logger,\n  level: :info\n\nconfig :rambla,\n  log_level: :debug,  # More detailed logging for Rambla\n  http: [\n    channels: [\n      chan_1: [\n        options: [\n          log: true,  # Log HTTP requests\n          log_level: :debug\n        ]\n      ]\n    ]\n  ]\n```","ref":"configuration.html#4-configure-logging"},{"type":"extras","title":"5. Testing Configuration - Configuration Guide","doc":"Use mock handlers in test environment:\n\n```elixir\n# config/test.exs\nconfig :rambla,\n  mock: [\n    connections: [mocked: :conn_mocked],\n    channels: [chan_1: [connection: :mocked]]\n  ],\n  stub: [\n    connections: [stubbed: :conn_stubbed],\n    channels: [chan_2: [connection: :stubbed]]\n  ]\n```","ref":"configuration.html#5-testing-configuration"},{"type":"extras","title":"6. Resource Management - Configuration Guide","doc":"Configure pool sizes based on resources:\n\n```elixir\nconfig :rambla,\n  redis: [\n    connections: [\n      local_conn: [\n        count: System.schedulers_online(),  # Match CPU cores\n        max_overflow: 10,\n        worker_options: [\n          timeout: 5000,\n          max_retries: 3\n        ]\n      ]\n    ]\n  ]\n```\n\nRemember to adjust these configurations based on your specific needs and environment requirements.","ref":"configuration.html#6-resource-management"},{"type":"extras","title":"Version Compatibility - Configuration Guide","doc":"","ref":"configuration.html#version-compatibility"},{"type":"extras","title":"Rambla 1.3.x - Configuration Guide","doc":"- All features documented here\n- Runtime configuration support\n- Enhanced error handling","ref":"configuration.html#rambla-1-3-x"},{"type":"extras","title":"Rambla 1.2.x - Configuration Guide","doc":"- Basic features\n- ClickHouse backend support\n- Support for callbacks to control execution","ref":"configuration.html#rambla-1-2-x"},{"type":"extras","title":"Rambla 1.1.x and earlier - Configuration Guide","doc":"- Limited backend support\n- Different configuration structure\n- Consider upgrading for new features","ref":"configuration.html#rambla-1-1-x-and-earlier"},{"type":"extras","title":"Troubleshooting Configuration - Configuration Guide","doc":"","ref":"configuration.html#troubleshooting-configuration"},{"type":"extras","title":"Common Configuration Issues - Configuration Guide","doc":"1. **Missing Dependencies**\n   ```elixir\n   ** (Mix.Error) Could not start application rambla: could not find application :amqp\n   ```\n   Solution: Add required dependencies to mix.exs:\n   ```elixir\n   {:amqp, \"~> 3.0\", optional: true}\n   ```\n\n2. **Invalid Configuration**\n   ```elixir\n   ** (ArgumentError) invalid configuration for :rambla\n   ```\n   Solution: Verify configuration structure matches the examples in this guide.\n\n3. **Runtime Errors**\n   ```elixir\n   ** (RuntimeError) no connection configured for :channel_name\n   ```\n   Solution: Check channel and connection configurations match.","ref":"configuration.html#common-configuration-issues"},{"type":"extras","title":"Monitoring and Metrics - Configuration Guide","doc":"Rambla supports telemetry events:\n\n```elixir\nconfig :telemetry,\n  events: [\n    [:rambla, :publish, :start],\n    [:rambla, :publish, :stop],\n    [:rambla, :publish, :exception]\n  ]\n```\n\nFor more details on implementing backends, see the [Custom Backends Guide](custom_backends.md) and for backend-specific options, see the [Backends Documentation](backends.md).","ref":"configuration.html#monitoring-and-metrics"},{"type":"extras","title":"Custom Backends","doc":"# Custom Backends\n\nThis guide explains how to create custom backends for Rambla by implementing the required behaviours and callbacks.","ref":"custom_backends.html"},{"type":"extras","title":"Documentation Index - Custom Backends","doc":"- [README](../README.md) - Overview and Quick Start\n- [Backend Documentation](backends.md) - Detailed backend guide\n- [Custom Backends](custom_backends.md) - Creating custom backends\n- [Configuration Reference](configuration.md) - Complete configuration guide","ref":"custom_backends.html#documentation-index"},{"type":"extras","title":"Table of Contents - Custom Backends","doc":"- [Basic Structure](#basic-structure)\n- [Required Callbacks](#required-callbacks)\n- [Configuration](#configuration)\n- [Error Handling](#error-handling)\n- [Complete Example](#complete-example)","ref":"custom_backends.html#table-of-contents"},{"type":"extras","title":"Basic Structure - Custom Backends","doc":"To create a custom backend, you need to:\n\n1. Create a new module under your application\n2. Use the `Rambla.Handler` behaviour\n3. Implement the required callbacks\n4. Configure your backend\n\nHere's the basic structure:\n\n```elixir\ndefmodule MyApp.Handlers.Custom do\n  @moduledoc \"\"\"\n  Custom backend implementation for MyApp.\n  \"\"\"\n  use Rambla.Handler\n\n  # Required callbacks will go here\nend\n```","ref":"custom_backends.html#basic-structure"},{"type":"extras","title":"Required Callbacks - Custom Backends","doc":"","ref":"custom_backends.html#required-callbacks"},{"type":"extras","title":"handle_publish/3 - Custom Backends","doc":"The main callback that handles message publishing. It receives:\n\n- `payload` - The message to be published\n- `options` - Publishing options\n- `state` - The current handler state\n\n```elixir\n@impl Rambla.Handler\ndef handle_publish(payload, options, state)\n\n# For direct message handling\ndef handle_publish(%{message: message}, options, %{connection: %{channel: name}}) do\n  # Handle the message\n  case publish_to_my_service(message, options) do\n    :ok -> {:ok, message}\n    {:error, reason} -> {:error, reason}\n  end\nend\n\n# For function-based handling\ndef handle_publish(callback, options, %{connection: %{channel: name}})\n    when is_function(callback, 1) do\n  callback.(source: __MODULE__, destination: name, options: options)\nend\n\n# Fallback for raw messages\ndef handle_publish(payload, options, state) do\n  handle_publish(%{message: payload}, options, state)\nend\n```","ref":"custom_backends.html#handle_publish-3"},{"type":"extras","title":"config/0 - Custom Backends","doc":"Returns the configuration structure for your backend:\n\n```elixir\n@impl Rambla.Handler\ndef config do\n  Application.get_env(:my_app, :my_backend, [])\nend\n```","ref":"custom_backends.html#config-0"},{"type":"extras","title":"external_servers/1 (optional) - Custom Backends","doc":"If your backend needs additional services to be started before the pool:\n\n```elixir\n@impl Rambla.Handler\ndef external_servers(channel) do\n  [\n    {MyApp.Service, name: service_name(channel)}\n  ]\nend\n```","ref":"custom_backends.html#external_servers-1-optional"},{"type":"extras","title":"Configuration - Custom Backends","doc":"Your backend should follow Rambla's configuration pattern:\n\n```elixir\n# config/config.exs\nconfig :my_app, :my_backend,\n  connections: [\n    default_conn: [\n      host: \"example.com\",\n      port: 1234,\n      # other connection options\n    ]\n  ],\n  channels: [\n    chan_1: [\n      connection: :default_conn,\n      options: [\n        # channel-specific options\n      ]\n    ]\n  ]\n```","ref":"custom_backends.html#configuration"},{"type":"extras","title":"Error Handling - Custom Backends","doc":"Rambla provides built-in error handling and retry mechanisms. Your `handle_publish/3` implementation should return:\n\n- `{:ok, result}` - Successful publish\n- `:ok` - Successful publish without result\n- `{:error, reason}` - Failed publish with reason\n- `:error` - Failed publish without specific reason\n\nThe handler will automatically:\n\n1. Retry failed operations (up to configured max_retries)\n2. Call success/failure callbacks\n3. Log appropriate messages","ref":"custom_backends.html#error-handling"},{"type":"extras","title":"Complete Example - Custom Backends","doc":"Here's a complete example of a custom backend:\n\n```elixir\ndefmodule MyApp.Handlers.Custom do\n  @moduledoc \"\"\"\n  Custom backend implementation for MyApp.\n  \"\"\"\n  use Rambla.Handler\n  require Logger\n\n  @impl Rambla.Handler\n  def handle_publish(%{message: message}, options, %{connection: %{channel: name}}) do\n    %{host: host, port: port} = get_in(options, [:connection, :params])\n    \n    case MyService.publish(host, port, message) do\n      :ok -> \n        {:ok, message}\n      {:error, reason} -> \n        {:error, \"Failed to publish: #{inspect(reason)}\"}\n    end\n  end\n\n  def handle_publish(callback, options, %{connection: %{channel: name}})\n      when is_function(callback, 1) do\n    callback.(source: __MODULE__, destination: name, options: options)\n  end\n\n  def handle_publish(payload, options, state) do\n    handle_publish(%{message: payload}, options, state)\n  end\n\n  @impl Rambla.Handler\n  def config do\n    [\n      connections: [\n        default: [\n          host: \"localhost\",\n          port: 4000,\n          timeout: 5000\n        ]\n      ],\n      channels: [\n        chan_1: [\n          connection: :default,\n          options: [\n            retries: 3,\n            callbacks: [\n              on_success: &handle_success/1,\n              on_failure: &handle_failure/1\n            ]\n          ]\n        ]\n      ]\n    ]\n  end\n\n  @impl Rambla.Handler\n  def external_servers(channel) do\n    [{MyApp.Service, name: service_name(channel)}]\n  end\n\n  # Optional callback implementations\n  @impl Rambla.Handler\n  def on_fatal(id, {nil, error}) do\n    Logger.error(\"[MyBackend] Fatal error on #{id}: #{inspect(error)}\")\n    super(id, {nil, error})\n  end\n\n  # Private functions\n  defp handle_success(%{id: id, outcome: result}) do\n    Logger.info(\"[MyBackend] Successfully published on #{id}: #{inspect(result)}\")\n    :ok\n  end\n\n  defp handle_failure(%{id: id, outcome: error}) do\n    Logger.warning(\"[MyBackend] Failed to publish on #{id}: #{inspect(error)}\")\n    :retry\n  end\n\n  defp service_name(channel) do\n    Module.concat([MyApp.Service, channel])\n  end\nend\n```","ref":"custom_backends.html#complete-example"},{"type":"extras","title":"Usage - Custom Backends","doc":"Once implemented, configure your backend in your application:\n\n```elixir\n# config/config.exs\nconfig :my_app, :my_backend,\n  connections: [\n    local: [\n      host: \"localhost\",\n      port: 4000\n    ]\n  ],\n  channels: [\n    chan_1: [\n      connection: :local,\n      options: [\n        timeout: 5000\n      ]\n    ]\n  ]\n\n# Add to your application supervisor\nchildren = [\n  MyApp.Handlers.Custom\n]\n```\n\nThen use it like any other Rambla backend:\n\n```elixir\nRambla.publish(:chan_1, %{my: \"message\"})\n```","ref":"custom_backends.html#usage"},{"type":"extras","title":"Integration with Existing Systems - Custom Backends","doc":"Your custom backend can integrate with any external system:\n\n- Message queues\n- Databases\n- APIs\n- File systems\n- Custom protocols\n- Internal services\n\nJust implement the appropriate client code in your `handle_publish/3` callback.","ref":"custom_backends.html#integration-with-existing-systems"},{"type":"extras","title":"Testing - Custom Backends","doc":"Create test helpers for your backend:\n\n```elixir\ndefmodule MyApp.Handlers.CustomTest do\n  use ExUnit.Case\n  \n  setup do\n    start_supervised!(MyApp.Handlers.Custom)\n    :ok\n  end\n  \n  test \"publishes message successfully\" do\n    assert :ok = Rambla.publish(:test_chan, %{test: true})\n  end\nend\n```\n\nFor integration testing, consider implementing a mock version of your backend using `Rambla.Handlers.Mock`.","ref":"custom_backends.html#testing"},{"type":"extras","title":"Troubleshooting Custom Backends - Custom Backends","doc":"Common implementation issues:\n\n1. **Pool Configuration**\n   - Ensure `external_servers/1` returns correct child specs\n   - Verify pool size settings\n   - Check worker initialization\n\n2. **Message Handling**\n   - Validate message format conversion\n   - Handle all error cases\n   - Implement proper cleanup\n\n3. **Integration Issues**\n   - Check service dependencies\n   - Verify connection parameters\n   - Test failure scenarios\n\n4. **Common Errors**\n   - `undefined function handle_publish/3` - Ensure you've implemented all required callbacks\n   - `no function clause matching` - Check pattern matching in your handle_publish functions\n   - `no process` - Verify your external service is running before the handler starts\n\nSee the [Configuration Guide](configuration.md) for detailed settings and [Backends Documentation](backends.md) for example implementations.","ref":"custom_backends.html#troubleshooting-custom-backends"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.1"}}